from rest_framework import serializers

from .mixins import BridgerSerializerFieldMixin
from .types import BridgerType, ReturnContentType


class JSONField(BridgerSerializerFieldMixin, serializers.JSONField):
    field_type = BridgerType.JSON.value


class JSONTextEditorField(BridgerSerializerFieldMixin, serializers.JSONField):
    field_type = BridgerType.TEXTEDITOR.value
    texteditor_content_type = ReturnContentType.JSON.value

    def get_representation(self, request, field_name):
        representation = super().get_representation(request, field_name)
        representation["content_type"] = self.texteditor_content_type
        return representation



from rest_framework.reverse import reverse

from bridger.buttons import ActionButton, ButtonConfig
from bridger.display import InstanceDisplay, Section, FieldSet
from bridger.settings import bridger_settings


def share_action_button(request):
    serializer = bridger_settings.DEFAULT_SHARE_SERIALIZER
    fields = [field for field in serializer().fields]

    btn = ActionButton(
        icon="wb-icon-trade",
        description_fields="",
        endpoint=reverse("bridger:share", request=request),
        instance_display=InstanceDisplay(
            sections=[Section(fields=FieldSet(fields=fields))]
        ),
        confirm_config=ButtonConfig(label="Share", icon="wb-icon-trade"),
        serializer=serializer,
    )
    btn.request = request
    return btn



from typing import List

from django.contrib.contenttypes.models import ContentType
from rest_framework.request import Request

from bridger.enums import Button
from bridger.metadata.mixins import BridgerMetadataMixin


class ButtonMetadata(BridgerMetadataMixin):
    key = "buttons"
    method_name = "_get_buttons"


class ButtonMetadataMixin:
    def get_instance_buttons(
        self, request: Request, buttons: List[str] = None
    ) -> List[str]:
        if buttons:
            return buttons

        button_list = list()

        if hasattr(self, "get_serializer_class"):
            serializer_class = self.get_serializer_class()
            if hasattr(serializer_class, "Meta"):
                model = self.get_serializer_class().Meta.model
                ct = ContentType.objects.get_for_model(model)

                if request.user.has_perm(f"{ct.app_label}.change_{ct.model}"):
                    button_list.append(Button.SAVE.value)
                if request.user.has_perm(f"{ct.app_label}.delete_{ct.model}"):
                    button_list.append(Button.DELETE.value)

        return button_list

    def _get_instance_buttons(self, request: Request) -> List[str]:
        return self.get_instance_buttons(
            request=request, buttons=getattr(self, "INSTANCE_BUTTONS", None)
        )

    def get_list_buttons(
        self, request: Request, buttons: List[str] = None
    ) -> List[str]:
        if buttons:
            return buttons

        button_list = list()

        if hasattr(self, "get_serializer_class"):
            serializer_class = self.get_serializer_class()
            if hasattr(serializer_class, "Meta"):
                model = self.get_serializer_class().Meta.model
                ct = ContentType.objects.get_for_model(model)

                if f"{ct.app_label}.add_{ct.model}":
                    button_list.append(Button.NEW.value)

        return button_list

    def _get_list_buttons(self, request: Request) -> List[str]:
        return self.get_list_buttons(
            request=request, buttons=getattr(self, "LIST_BUTTONS", None)
        )

    def get_buttons(self, request: Request, buttons: List[str] = None) -> List[str]:
        if buttons:
            return buttons

        button_list = list()

        pk = self.kwargs.get("pk", None)

        if pk:
            button_list.extend(self._get_instance_buttons(request))
        else:
            button_list.extend(self._get_list_buttons(request))

        button_list.append(Button.REFRESH.value)

        return set(button_list)

    def _get_buttons(self, request: Request) -> str:
        return self.get_buttons(request=request, buttons=getattr(self, "BUTTONS", None))



from rest_framework.permissions import IsAuthenticated
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.views import APIView
from rest_framework import status

from markdown_blockdiag.utils import draw_blockdiag, DIAG_MODULES
from blockdiag.parser import ParseException


class BlockDiag(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request: Request) -> Response:
        try:
            blockdiag = request.data["markdown"]
            svg = draw_blockdiag(blockdiag, output_fmt="svg", font_antialias=True)
        except (ParseException, KeyError):
            return Response(status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response(svg)



from datetime import datetime, date, timedelta

from django.conf import settings


def get_date_interval_from_get(request):
    """
    Parses a request and returns the start and end date from it.

    Parameters
    ----------
    request: The GET Request Object

    Returns
    -------
    Return a tuple in the form of (start_date, end_date). If either the start date or the end date is not present in the request None is returned in the tuple
    """
    print(request)
    start_identifier = getattr(
        settings,
        "BRIDGER_START_IDENTIFIERS",
        ["start", "start_date", "from", "date_gte"],
    )
    end_identifier = getattr(
        settings, "BRIDGER_END_IDENTIFIERS", ["end", "end_date", "to", "date_lte"]
    )
    date_format = getattr(settings, "BRIDGER_DATE_FORMAT", "%Y-%m-%d")

    assert isinstance(start_identifier, list)
    assert isinstance(end_identifier, list)
    assert isinstance(date_format, str)

    start = next(
        (identifier for identifier in start_identifier if identifier in request.GET),
        None,
    )
    end = next(
        (identifier for identifier in end_identifier if identifier in request.GET), None
    )

    if start:
        try:
            start = datetime.strptime(request.GET.get(start), date_format).date()
        except ValueError:
            start = None
    if end:
        try:
            end = datetime.strptime(request.GET.get(end), date_format).date()
        except ValueError:
            end = None

    return start, end


def get_quarter_from_date(d):
    return ((d.month - 1) // 3) + 1


def get_start_date_from_date(d):
    quarter = get_quarter_from_date(d)
    return date(d.year, quarter * 3 - 2, 1)


def get_end_date_from_date(d):
    quarter = get_quarter_from_date(d)
    return date(
        d.year + ((quarter * 3 + 1) // 12), (quarter * 3 + 1) % 12, 1
    ) - timedelta(days=1)


def get_start_and_end_date_from_date(d):
    return get_start_date_from_date(d), get_end_date_from_date(d)


from django.utils import timezone
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.reverse import reverse

from bridger import buttons as bt
from bridger import display as dp
from bridger import viewsets
from bridger.enums import RequestType
from django.db.models.functions import Concat
from django.db.models import F, Value
from .serializers import get_historical_serializer
from bridger.enums import Button


def get_historical_viewset(model, historical_model):
    class HistoricalModelViewSet(viewsets.ModelViewSet):

        BUTTONS = [Button.REFRESH.value]
        CREATE_BUTTONS = []

        def get_custom_instance_buttons(self, request, buttons):
            return []

        def get_custom_list_instance_buttons(self, request, buttons):
            return []

        LIST_DISPLAY = dp.ListDisplay(
            fields=[
                dp.Field(key="history_date", label="Changed"),
                dp.Field(key="history_change_reason", label="Reason"),
                dp.Field(key="history_type", label="Type"),
                dp.Field(key="user_repr", label="User"),
            ]
        )

        def get_instance_endpoint(self, request, endpoint=None):
            return (
                f"{model.get_endpoint_basename()}-history-list",
                [self.kwargs["model_pk"]],
                {},
            )

        queryset = historical_model.objects.all()
        serializer_class = get_historical_serializer(historical_model)

        ordering_fields = ("history_id",)

        def get_queryset(self):
            qs = super().get_queryset()
            qs = qs.annotate(
                user_repr=Concat(
                    F("history_user__first_name"),
                    Value(" "),
                    F("history_user__last_name"),
                )
            )
            return qs

    return HistoricalModelViewSet




from rest_framework import filters
from rest_framework.request import Request

from bridger.metadata.mixins import BridgerMetadataMixin


class OrderingFieldsMetadata(BridgerMetadataMixin):
    key = "ordering_fields"
    method_name = "_get_ordering_fields"


class OrderingFieldsMetadataMixin:
    def _get_ordering_fields(self, request: Request):
        if filters.OrderingFilter in getattr(self, "filter_backends", []):
            yield from self.ordering_fields



from typing import Dict

from datetime import timedelta

from django.contrib.auth import get_user_model
from django.conf import settings
from rest_framework.request import Request
from rest_framework.reverse import reverse


def unauthenticated(request: Request) -> Dict:
    return {"type": None}


def jwt_auth(request: Request) -> Dict:
    user_model = get_user_model()
    username_field_key = user_model.USERNAME_FIELD
    username_field_label = user_model._meta.get_field(username_field_key).verbose_name

    access = settings.SIMPLE_JWT.get("ACCESS_TOKEN_LIFETIME", timedelta(minutes=5))
    refresh = settings.SIMPLE_JWT.get("REFRESH_TOKEN_LIFETIME", timedelta(days=1))

    return {
        "type": "JWT",
        "config": {
            "token": reverse("token_obtain_pair", request=request),
            "refresh": reverse("token_refresh", request=request),
            "verify": reverse("token_verify", request=request),
            "token_lifetime": {"access": access, "refresh": refresh},
            "username_field_key": username_field_key,
            "username_field_label": username_field_label,
        },
    }



from django.contrib.contenttypes.models import ContentType
from rest_framework.request import Request

from bridger.metadata.mixins import BridgerMetadataMixin


class IdentifierMetadata(BridgerMetadataMixin):
    key = "identifier"
    method_name = "_get_identifier"


class IdentifierMetadataMixin:
    def get_identifier(self, request: Request, identifier: str = None) -> str:
        assert identifier is not None or hasattr(
            self, "get_serializer_class"
        ), "Viewsets without serializer classes need to set IDENTIFIER or override get_identifier"

        return identifier or "{0.app_label}:{0.model}".format(
            ContentType.objects.get_for_model(self.get_serializer_class().Meta.model)
        )

    def _get_identifier(self, request: Request) -> str:
        return self.get_identifier(
            request=request, identifier=getattr(self, "IDENTIFIER", None)
        )



from django.utils import timezone
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.reverse import reverse

from bridger import buttons as bt
from bridger import display as dp
from bridger import viewsets
from bridger.enums import RequestType

from .models import Notification
from .serializers import NotificationModelSerializer


class NotificationModelViewSet(viewsets.ModelViewSet):
    ENDPOINT = "bridger:notification-list"

    INSTANCE_TITLE = "Notification: {{title}}"
    LIST_TITLE = "Notifications"

    LIST_BUTTONS = ["refresh"]
    INSTANCE_BUTTONS = ["refresh", "delete"]
    CREATE_BUTTONS = []

    def get_custom_buttons(self, request, buttons):
        return [
            bt.ActionButton(
                method=RequestType.POST,
                action_label="All notifications read.",
                endpoint=reverse(
                    "bridger:notification-mark-all-as-read", request=request
                ),
                description_fields="Do you want to mark notifications as read?",
                label="Mark all as read",
                icon="wb-icon-eye-open",
                confirm_config=bt.ButtonConfig(label="Read all"),
                cancel_config=bt.ButtonConfig(label="Cancel"),
                identifiers=["relatedmodeltest-list"],
            ),
            bt.ActionButton(
                method=RequestType.POST,
                action_label="Delete all read notifications.",
                endpoint=reverse(
                    "bridger:notification-delete-all-read", request=request
                ),
                description_fields="Do you want delete all read notifications?",
                label="Delete all read notifications",
                icon="wb-icon-trash",
                confirm_config=bt.ButtonConfig(
                    label="Delete all", level=bt.ButtonLevel.WARNING
                ),
                cancel_config=bt.ButtonConfig(
                    label="Cancel", level=bt.ButtonLevel.ERROR
                ),
                identifiers=[reverse("bridger:notification-list", request=request)],
            ),
        ]

    LIST_DISPLAY = dp.ListDisplay(
        fields=[
            dp.Field(key="title", label="Title"),
            dp.Field(key="timestamp_created", label="Created"),
            dp.Field(key="message", label="Message"),
        ],
        formatting=[
            dp.Formatting(
                column="timestamp_read",
                formatting_rules=[
                    dp.FormattingRule(icon="wb-icon-eye-open", condition=("∃", True)),
                ],
            ),
        ],
    )

    INSTANCE_DISPLAY = dp.InstanceDisplay(
        sections=[
            dp.Section(
                fields=dp.FieldSet(
                    fields=[
                        dp.FieldSet(fields=["timestamp_created", "timestamp_received"]),
                        dp.FieldSet(fields=["timestamp_received", "timestamp_mailed"]),
                        "title",
                        "message",
                    ]
                )
            )
        ]
    )

    queryset = Notification.objects.all()
    serializer_class = NotificationModelSerializer

    ordering_fields = ("timestamp_created",)

    def retrieve(self, request, *args, **kwargs):
        obj = self.get_object()
        if not obj.timestamp_read:
            obj.timestamp_read = timezone.now()
            obj.save()
        return super().retrieve(request, *args, **kwargs)

    def list(self, request, *args, **kwargs):
        self.get_queryset().filter(timestamp_received__isnull=True).update(
            timestamp_received=timezone.now()
        )
        return super().list(request, *args, **kwargs)

    def get_queryset(self):
        return super().get_queryset().filter(recipient=self.request.user)

    @action(methods=["POST"], detail=False)
    def mark_all_as_read(self, request, pk=None):
        Notification.objects.filter(timestamp_read__isnull=True).update(
            timestamp_read=timezone.now()
        )
        return Response({})

    @action(methods=["POST"], detail=False)
    def delete_all_read(self, request, pk=None):
        Notification.objects.filter(timestamp_read__isnull=False).delete()
        return Response({})



from .fields import (
    ButtonMetadataMixin,
    CreateButtonMetadataMixin,
    CustomButtonMetadataMixin,
    CustomInstanceButtonMetadataMixin,
    EndpointMetadataMixin,
    FieldsMetadataMixin,
    FilterFieldsMetadataMixin,
    IdentifierMetadataMixin,
    InstanceDisplayMetadataMixin,
    ListDisplayMetadataMixin,
    OrderingFieldsMetadataMixin,
    PaginationMetadataMixin,
    PKMetadataMixin,
    SearchFieldsMetadataMixin,
    TitleMetadataMixin,
    WidgetTypeMetadataMixin,
    PreviewMetadataMixin,
    DocsMetadataMixin,
)
from .metadata import BridgerMetadata


class MetadataMixin(
    InstanceDisplayMetadataMixin,
    ListDisplayMetadataMixin,
    IdentifierMetadataMixin,
    WidgetTypeMetadataMixin,
    EndpointMetadataMixin,
    ButtonMetadataMixin,
    CreateButtonMetadataMixin,
    CustomButtonMetadataMixin,
    CustomInstanceButtonMetadataMixin,
    PaginationMetadataMixin,
    TitleMetadataMixin,
    SearchFieldsMetadataMixin,
    OrderingFieldsMetadataMixin,
    FilterFieldsMetadataMixin,
    FieldsMetadataMixin,
    PKMetadataMixin,
    PreviewMetadataMixin,
    DocsMetadataMixin,
):
    metadata_class = BridgerMetadata



class BridgerMetadataMixin:
    def __init__(self, view, request, *args, **kwargs):
        self.view = view
        self.request = request
        super().__init__(*args, **kwargs)

    def __iter__(self):
        yield self.key, getattr(self.view, self.method_name)(self.request)



from rest_framework import serializers

from .mixins import BridgerSerializerFieldMixin
from .types import BridgerType


class DateTimeField(BridgerSerializerFieldMixin, serializers.DateTimeField):
    field_type = BridgerType.DATETIME.value
    # format = "%Y-%m-%d"


class DateField(BridgerSerializerFieldMixin, serializers.DateField):
    field_type = BridgerType.DATE.value


class TimeField(BridgerSerializerFieldMixin, serializers.TimeField):
    field_type = BridgerType.TIME.value



from contextlib import suppress

from rest_framework.filters import OrderingFilter, SearchFilter
from rest_framework.mixins import CreateModelMixin as OriginalCreateModelMixin
from rest_framework.mixins import DestroyModelMixin
from rest_framework.mixins import ListModelMixin as OriginalListModelMixin
from rest_framework.mixins import RetrieveModelMixin as OriginalRetrieveModelMixin
from rest_framework.mixins import UpdateModelMixin as OriginalUpdateModelMixin
from rest_framework.request import Request
from rest_framework.response import Response

from bridger.filters import DjangoFilterBackend
from bridger.messages import serialize_messages


class FilterMixin:
    filter_backends = (OrderingFilter, SearchFilter, DjangoFilterBackend)
    filter_fields = {}
    search_fields = []
    ordering_fields = ordering = ("id",)


class ModelMixin:
    @classmethod
    def get_model(cls):
        try:
            if hasattr(cls, "queryset"):
                return cls.queryset.model
            elif hasattr(cls, "serializer_class"):
                return cls.serializer_class.Meta.model
            else:
                return None
        except AttributeError:
            return None


class ListModelMixin(OriginalListModelMixin):
    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)

        if page is not None:
            serializer = self.get_serializer(page, many=True)
            response = self.get_paginated_response(serializer.data)
            with suppress(TypeError, AttributeError):
                messages = self.get_messages(
                    request=request,
                    queryset=queryset,
                    paginated_queryset=page,
                    initial=self._paginator.is_initial(),
                )
                response.data["messages"] = serialize_messages(messages)

            with suppress(TypeError, AttributeError):
                response.data["aggregates"] = self.get_aggregates(
                    queryset=queryset, paginated_queryset=page
                )
        else:
            serializer = self.get_serializer(queryset, many=True)
            response = Response(serializer.data)
            response.data = {"results": response.data}
            with suppress(TypeError, AttributeError):
                messages = self.get_messages(
                    request=request, queryset=queryset, initial=True
                )
                response.data["messages"] = serialize_messages(messages)

            with suppress(TypeError, AttributeError):
                response.data["aggregates"] = self.get_aggregates(queryset=queryset)

        return response


class RetrieveModelMixin(OriginalRetrieveModelMixin):
    def retrieve(self, request, *args, **kwargs):
        response = super().retrieve(request, *args, **kwargs)
        response.data = {"instance": response.data}

        with suppress(AttributeError, TypeError):
            messages = self.get_messages(request=request, instance=self.get_object())
            response.data["messages"] = [dict(message) for message in messages]

        return response


class CreateModelMixin(OriginalCreateModelMixin):
    def create(self, request, *args, **kwargs):
        response = super().create(request, *args, **kwargs)
        response.data = {"instance": response.data}
        return response


class UpdateModelMixin(OriginalUpdateModelMixin):
    def update(self, request, *args, **kwargs):
        response = super().update(request, *args, **kwargs)
        response.data = {"instance": response.data}
        return response


class DestroyMultipleModelMixin:
    def destroy_multiple(self, request, *args, **kwargs):
        model = self.get_serializer_class().Meta.model
        app_label = model._meta.app_label

        queryset = model.objects.filter(id__in=request.data)
        destroyed = self.perform_destroy_multiple(queryset)

        return Response(
            {"count": destroyed[1].get(f"{app_label}.{model.__name__}", 0)},
            status=status.HTTP_204_NO_CONTENT,
        )

    def perform_destroy_multiple(self, queryset):
        return queryset.delete()



from typing import Dict, Union

from rest_framework.request import Request

from bridger.metadata.mixins import BridgerMetadataMixin
from bridger.serializers import (
    ListSerializer,
    RepresentationSerializer,
    percent_decorator,
)


class FieldsMetadata(BridgerMetadataMixin):
    key = "fields"
    method_name = "_get_fields"


class FieldsMetadataMixin:
    def get_fields(self, request: Request) -> Dict:
        fields = dict()
        rs = RepresentationSerializer
        ls = ListSerializer
        field_items = self.get_serializer().fields.items()

        for name, field in filter(
            lambda f: not isinstance(f[1], (rs, ls)), field_items
        ):
            fields[name] = field.get_representation(request, name)

        for name, field in filter(lambda f: isinstance(f[1], (rs, ls)), field_items):
            representation = field.get_representation(request, name)
            fields[representation["related_key"]].update(representation)
            del fields[representation["related_key"]]["related_key"]

        return fields

    def _get_fields(self, request: Request) -> Dict:
        fields = self.get_fields(request)
        serializer_class = self.get_serializer_class()

        for key, value in serializer_class.get_decorators():
            fields[key]["decorators"].append(value)
        for key in serializer_class.get_percent_fields():
            fields[key]["type"] = "percent"
            fields[key]["decorators"].append(percent_decorator)

        return fields



from django.urls import include, path
from django.views.generic import TemplateView

from .frontend_user_configuration import FrontendUserConfigurationModelViewSet
from .notifications.viewsets import NotificationModelViewSet
from .routers import BridgerRouter
from .views import Config, Profile

from .share.views import ShareAPIView
from .menus.views import MenuAPIView
from .profile import UserViewSet
from .markdown.views import BlockDiag
from .clubhouse import ClubHouseView

# fmt: off
router = BridgerRouter()
router.register(r"frontenduserconfiguration", FrontendUserConfigurationModelViewSet, basename="frontenduserconfiguration")
router.register(r"notification", NotificationModelViewSet, basename="notification")
router.register(r"user", UserViewSet, basename="user")
router.register(r"clubhouse", ClubHouseView, basename="clubhouse")
# fmt: on

urlpatterns = [
    path("config/", Config.as_view(), name="config"),
    path("profile/", Profile.as_view(), name="profile"),
    path("menu/", MenuAPIView.as_view(), name="menu"),
    path("share/", ShareAPIView.as_view(), name="share"),
    path("markdown/blockdiag/", BlockDiag.as_view(), name="blockdiag"),
    path("", include(router.urls)),
]



import operator
from functools import reduce

from django.db.models import F, Q, QuerySet, Value
from django.db.models.functions import Concat

search_models = []


class SearchRegistry:
    def __init__(self):
        self._registry = {}

    def register(self, _class, endpoint):
        self._registry[_class.__name__] = {
            "class": _class,
            "verbose_name": _class._meta.verbose_name,
            "endpoint": endpoint,
        }

    def registered_models(self):
        yield from self._registry.items()


registry = SearchRegistry()


def register(endpoint):
    def _search_wrapper(model_class):
        registry.register(model_class, endpoint)
        return model_class

    return _search_wrapper


def search(search_term, request=None):
    qs = None
    for key, values in registry._registry.items():

        # filtering = reduce(
        #     lambda x, y: operator.and_(
        #         Q(__search__icontains=x), Q(__search__icontains=y)
        #     ),
        #     search_term.split(" "),
        # )

        _qs = (
            values["class"]
            .search_for_term(search_term, request)
            .annotate(
                __search=Concat(F("_search"), Value(f" {values['verbose_name']}"))
            )
        )

        for term in search_term.split(" "):
            _qs = _qs.filter(__search__icontains=term)

        _qs = _qs.values("id", "_repr", "__search")

        if qs is None:
            qs = _qs
        else:
            qs = qs.union(_qs)

    return qs



# TODO: Most likely to be removed
from rest_framework import serializers

from .mixins import BridgerSerializerFieldMixin
from .number import IntegerField
from .types import BridgerType, ReturnContentType


class StarRatingField(IntegerField):
    field_type = "starrating"


class RangeSelectField(BridgerSerializerFieldMixin, serializers.FloatField):
    field_type = "rangeselect"

    def __init__(self, *args, **kwargs):
        self.color = kwargs.pop("color", "rgb(133, 144, 162)")
        super().__init__(*args, **kwargs)

    def get_representation(self, request, field_name):
        representation = super().get_representation(request, field_name)
        representation["color"] = self.color
        return representation



from bridger.buttons import WidgetButton
from bridger import serializers

from .models import Notification


class NotificationModelSerializer(serializers.ModelSerializer):

    @serializers.register_dynamic_button()
    def notification_buttons(self, instance, request, user):
        btns = instance.buttons or []
        if instance.endpoint:
            btns.append(WidgetButton(
                endpoint=instance.endpoint,
                label="Open",
                title="Open",
                icon="wb-icon-data"
            ))
        return btns

    class Meta:
        model = Notification
        fields = read_only_fields = (
            "id",
            "recipient",
            "title",
            "message",
            "timestamp_created",
            "timestamp_received",
            "timestamp_read",
            "timestamp_mailed",
            "_buttons",
        )



# Generated by Django 2.2.12 on 2020-04-02 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bridger', '0004_auto_20200327_1152'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='endpoint',
            field=models.URLField(blank=True, null=True),
        ),
    ]



from typing import Dict, Union

from rest_framework.request import Request
from bridger.metadata.mixins import BridgerMetadataMixin


class PreviewMetadata(BridgerMetadataMixin):
    key = "preview"
    method_name = "_get_preview"


class PreviewMetadataMixin:

    def get_preview_buttons(self, request: Request, buttons):
        return buttons

    def get_preview_display(self, request: Request, display):
        return display

    def get_preview_type(self, request: Request, preview_type):
        return preview_type

    def _get_preview_display(self, request: Request, display, preview_type):
        if preview_type == "html":
            return display
        elif preview_type == "instance_display":
            return list(display)

    def _get_preview(self, request: Request) -> Dict:
        buttons = self.get_preview_buttons(
            request=request, buttons=getattr(self, "PREVIEW_BUTTONS", [])
        )
        button_list = list()

        for button in buttons:
            button_list.append(dict(button))

        preview_type = self.get_preview_type(request, getattr(self, "PREVIEW_TYPE", "html"))

        _display = self.get_preview_display(request, getattr(self, "PREVIEW_DISPLAY", None))
        display = self._get_preview_display(
            request=request, display=_display, preview_type=preview_type
        )

        return {
            "buttons": button_list,
            "display": display,
            "type": preview_type
        }